#include <stdio.h>
#include <dos.h>

typedef struct {
    unsigned char seconds;	/* AT Real Time Clock (RTC): Seconds */
    unsigned char secalrm;	/* AT RTC: Seconds Alarm */
    unsigned char minutes;	/* AT RTC: Minutes */
    unsigned char minalrm;	/* AT RTC: Minutes Alarm */
    unsigned char hours;	/* AT RTC: Hours */
    unsigned char hrsalrm;	/* AT RTC: Hours Alarm */
    unsigned char dayofweek;	/* AT RTC: day of week */
    unsigned char dayofmon;	/* AT RTC: day of month */
    unsigned char month;	/* AT RTC: month */
    unsigned char year;		/* AT RTC: year */
    unsigned char aregister;	/* STATUS REGISTER A */
    unsigned char bregister;	/* STATUS REGISTER B */
    unsigned char cregister;	/* STATUS REGISTER C */
    unsigned char dregister;	/* STATUS REGISTER D */
    unsigned char diagnostic;	/* Diagnostics status byte */
    unsigned char shutdown;	/* Shutdown status byte */
    unsigned char diskettes;	/* A & B diskette types */
    unsigned char reserved1;	/* undefined */
    unsigned char harddrive;	/* C & D hard drive types */
    unsigned char reserved2;	/* undefined */
    unsigned char equipment;	/* equipment byte */
    unsigned char lowbyte;	/* low byte of base memory */
    unsigned char highbyte;	/* high byte of base memory */
    /* 100h = 256k, 200h = 512k, 280h = 640k */
    unsigned char extlow;	/* low byte of extended memory */
    unsigned char exthigh;	/* high byte of extended memory */
    /* 200h=512k;400h=1024k;etc to 3c00h=15360k */
    unsigned char drivec;	/* more data on drive c */
    unsigned char drived;	/* more data on drive d */
    unsigned char reserved[19];	/* reserved */
    unsigned checksum;
    unsigned char extlow1;	/* same as extlow */
    unsigned char exthigh1;	/* same as exthigh */
    unsigned char century;	/* binary coded decimal value for century */
    /* 19h = 1900 for example */
    unsigned char infoflag;	/* bit 7 set = top 128k installed */
    unsigned char info[12];
} CMOS;

#define	CMOS_ADDR	0x70	/* address port of CMOS */
#define	CMOS_DATA	0x71	/* data port for CMOS */

void GetCMOS(unsigned char *cmosdata)
{				/* read CMOS data (64 bytes) */
    unsigned unsigned char j, byte;
    for (j = 0; j < 64; j++) {
	disable();		/* disable interrupts */
	outportb(CMOS_ADDR, j);	/* specify byte to get */
	byte = inportb(CMOS_DATA);	/* get data */
	enable();		/* enable interrupts */
	*cmosdata++ = byte;	/* save CMOS data */
    }
}

void main(void)
{
    CMOS cmosdata;
    unsigned char *iptr = (unsigned char *) &cmosdata;
    int j, k;

    GetCMOS(iptr);		/* read 64 bytes of CMOS data */
    printf("\nHex Dump of CMOS RAM:\n");
    for (j = 0, k = 0; j < 64; j++) {
	printf("%02x ", *iptr++);
	k++;
	if (k == 16) {
	    k = 0;
	    printf("\n");
	}
    }
}
