#include <dos.h>
#include <stdio.h>
#include <string.h>
#include <bios.h>
#include <stdlib.h>
#include <ctype.h>
#include <io.h>
#include <dos.h>
#include <time.h>
#include "sektor.h"

#ifdef MK_FP                      /* Macro MK_FP already defined? */
  #undef MK_FP                          /* Yes, then delete macro */
#endif

#define MK_FP(seg,ofs) ((void far *) ((unsigned long) (seg)<<16|( ofs)))
#define LO( aval ) ( ( BYTE ) ( aval & 0xFF ) )
#define HI( aval ) ( ( BYTE ) ( aval >> 8 ) )
#define SEG( p ) ( ( unsigned int ) ( ( ( long ) p ) >> 16 ) )
#define OFS( p ) ( ( unsigned int ) ( p ) )


#define NOPE         0x4E
#define NO_DRIVE     0
#define DD_525       1
#define HD_525       2
#define DD_35        3
#define HD_35        4

#define TRUE         ( 0 == 0 )
#define FALSE        ( 1 == 0 )

typedef unsigned char BYTE;

typedef BYTE DDPTType[ 11 ];
typedef DDPTType *DDPTPTR;

typedef struct {                    /* Physical format parameters */
		  BYTE    DSides,              /* Number of sides */
		       STrax,                  /* Tracks per side */
		       TSectors;             /* Sectors per track */
	       DDPTPTR DDPT;                  /* Pointer to DDPTR */
		} PhysDataType;

typedef struct {                     /* Logical format parameters */
	       BYTE Media;                       /* Media byte */
	       BYTE Cluster;     /* Number of sectors per cluster */
	     BYTE FAT;         /* Number of sectors for the FAT */
	       BYTE RootSize;    /* Entries in the root directory */
		} LogDataType;

typedef BYTE TrackBfType[ 18 ][ 1024 ];      /* Memory for a track */
char buf[1024];

void far *GetIntVec( int Number )
{
 return( *( ( void far * far * ) MK_FP( 0, Number * 4 ) ) );
}


void SetIntVec( int Number, void far *Pointer )
{
 *( ( void far * far * ) MK_FP( 0, Number * 4 ) ) = Pointer;
}


BYTE GetDriveType( BYTE Drive )
{
 union REGS regs;       /* Processor registers for interrupt call */

 regs.h.ah = 0x08;              /* Function: Determine drive type */
 regs.h.dl = Drive;                               /* Drive number */
 int86( 0x13, &regs, &regs );              /* Call BIOS interrupt */
 if ( regs.x.cflag )                            /* Error in call? */
  return( DD_525 );        /* Fct. 0x08 does not exist => 360K XT */
 else
  return( regs.h.bl );                              /* Drive type */
}


void DiskReset( void )
{
 union REGS regs;       /* Processor registers for interrupt call */

 regs.h.ah = 0x00;          /* Function number for interrupt call */
 regs.h.dl = 0;                            /* Drive a: (see Info) */
 int86( 0x13, &regs, &regs );                   /* Interrupt call */
}


BYTE GetFormatParameter( char         *FormString,
			 BYTE         DriveType,
			 PhysDataType *PDataP,
			 LogDataType  *LDataP )

{
 static DDPTType DDPT_360  = { 0xDF, 0x02, 0x25, 0x02, 0x09, 0x2A,
                              0xFF, 0x50, 0xF6, 0x0F, 0x08 };
 static DDPTType DDPT_1200 = { 0xDF, 0x02, 0x25, 0x02, 0x0F, 0x1B,
                              0xFF, 0x54, 0xF6, 0x0F, 0x08 };
 static DDPTType DDPT_1440 = { 0xDF, 0x02, 0x25, 0x01, 0x14, 0x1B,
			      0xFF, 0x6C, 0xF6, 0x0F, 0x08 };
 static DDPTType DDPT_720  = { 0xDF, 0x02, 0x25, 0x02, 0x09, 0x2A,
			      0xFF, 0x50, 0xF6, 0x0F, 0x08 };

 static LogDataType LOG_360  = { 0xFD, 2, 2, 0x70 };
 static LogDataType LOG_1200 = { 0xF9, 1, 7, 0xE0 };
 static LogDataType LOG_720  = { 0xF9, 2, 3, 0x70 };
 static LogDataType LOG_1440 = { 0xF0, 1, 9, 0xE0 };

 static PhysDataType PHYS_360  = { 2, 40,  9, &DDPT_360 };
 static PhysDataType PHYS_1200 = { 2, 80, 15, &DDPT_1200 };
 static PhysDataType PHYS_720  = { 2, 80,  9, &DDPT_720 };
 static PhysDataType PHYS_1440 = { 2, 80, 20, &DDPT_1440 };


 if ( strcmp( FormString, "1200" ) == 0 )    /* 1.2 Meg on 5.25"? */
   if ( DriveType == HD_525 )    /* Format compatible with drive? */
    {
     memcpy( PDataP, &PHYS_1200, sizeof( PhysDataType ) );
     memcpy( LDataP, &LOG_1200, sizeof ( LogDataType ) );
     return TRUE;                            /* End without error */
    }
    else
     return( FALSE );            /* Drive and format incompatible */
  else if ( strcmp( FormString, "360" ) == 0 )           /* 360K? */
   if ( ( DriveType == HD_525 ) || ( DriveType == DD_525 ) )
    {              /* Format and drive compatible, set parameters */
     memcpy ( PDataP, &PHYS_360, sizeof( PhysDataType ) );
     memcpy ( LDataP, &LOG_360, sizeof ( LogDataType ) );
     return TRUE;                            /* End without error */
    }
    else
     return( FALSE );            /* Drive and format incompatible */
  else if ( strcmp( FormString, "1440" ) == 0 ) /* 1.44 Meg on 3.5"? */
   if ( DriveType == HD_35 )     /* Format compatible with drive? */
    {              /* Format and drive compatible, set parameters */
     memcpy ( PDataP, &PHYS_1440, sizeof( PhysDataType ) );
     memcpy ( LDataP, &LOG_1440, sizeof ( LogDataType ) );
     return TRUE;                            /* End without error */
    }
    else
     return( FALSE );            /* Drive and format incompatible */
  else if ( strcmp( FormString, "720" ) == 0 )      /* 720K on 3.5"? */
    if ( ( DriveType == HD_35 ) || ( DriveType == DD_35 ) )
    {              /* Format and drive compatible, set parameters */
     memcpy ( PDataP, &PHYS_720, sizeof( PhysDataType ) );
     memcpy ( LDataP, &LOG_720, sizeof ( LogDataType ) );
     return TRUE;                            /* End without error */
    }
    else
     return FALSE;               /* Drive and format incompatible */
  else
    return FALSE;                     /* Invalid format specified */
}


void DiskPrepare( BYTE Drive, PhysDataType PData )
{
 union REGS regs;       /* Processor registers for interrupt call */

 regs.h.ah = 0x18;          /* Function number for interrupt call */
 regs.h.ch = PData.STrax - 1;        /* Number of tracks per side */
 regs.h.cl = PData.TSectors;       /* Number of sectors per track */
 regs.h.dl = Drive;                               /* Drive number */
 int86( 0x13, &regs, &regs );                   /* Interrupt call */
}


int protection(void)
{
 BYTE         CurDriveType;            /* Current disk drive type */
 PhysDataType PData;                /* Physical format parameters */
 LogDataType  LData;                 /* Logical format parameters */
 void far     *POldDDPT;                   /* Pointer to old DDPT */
 int s,ok2=0,ok3=0;


       randomize();
       s=random(19);
       sektor2[255]=s+2;
       s+=162;
       CurDriveType = GetDriveType( 0 );   /* Current drive type */
       GetFormatParameter( "1440", CurDriveType, &PData, &LData );
       DiskPrepare( 0, PData );
       POldDDPT = GetIntVec( 0x1E );            /* Store old DDPT */
       SetIntVec( 0x1E, PData.DDPT );             /* Set new DDPT */
       biosdisk(2,0,1,80,s,1,buf);
       biosdisk(2,0,1,80,s,1,buf);
       ok2=biosdisk(2,0,1,80,161,1,buf);
       SetIntVec( 0x1E, POldDDPT );           /* Restore old DDPT */

       if(_fmemcmp(sektor2,buf,256)) ok3=1;
       if(ok2!=4 || ok3!=0) return(1);
       return(0);
}
